@@ -1,9 +1,7 @@
-import datetime
-
-from typing import List, Optional, Any
-from typing.io import BinaryIO
+from typing import Any
 
 from pydantic import BaseModel, HttpUrl
+
 from .client import SDKClient, SDKResponse
 
 
@@ -18,20 +16,16 @@ class SignResponse(BaseModel):
 
 
 class CryptoProService:
-    def __init__(self, client: SDKClient, url: HttpUrl):
+    def __init__(self, client: SDKClient):
         self._client = client
-        self._url = url
 
     def sign(
-        self, query: SignRequest, token: str, timeout=3
+        self, query: SignRequest, token: str, url: HttpUrl, timeout=3
     ) -> SDKResponse[SignResponse]:
         return self._client.post(
-            self._url + "signer",
+            url + "signer",
             SignResponse,
-            params={
-                "thumbprint": query.thumbprint,
-                "api-key": token
-            },
+            params={"thumbprint": query.thumbprint, "api-key": token},
             files={"file": query.xml_file},
             timeout=timeout,
         )
