@@ -2,7 +2,7 @@ from datetime import date as datetime_date
 from enum import Enum, IntEnum
 from typing import List, Optional
 
-from pydantic import BaseModel, Field, HttpUrl
+from pydantic import BaseModel, Field, HttpUrl, root_validator
 
 from .client import Empty, SDKClient, SDKResponse
 
@@ -14,6 +14,8 @@ LONG_GIGTEST_TIMEOUT = 60 + 2
 class MedResearchType(IntEnum):
     exam = 0
     lab = 1
+    func = 2
+    vaccine = 3
 
 
 class MedResearchBlockType(Enum):
@@ -22,6 +24,26 @@ class MedResearchBlockType(Enum):
     vaccine = 2
 
 
+class MedBookType(IntEnum):
+    paper = 0
+    digital = 1
+
+
+class GenderType(IntEnum):
+    any = 0
+    male = 1
+    female = 2
+
+
+class SyncStatus(str, Enum):
+    awaiting_signs = "awaiting_signs"  # "Ждет подписания"
+    pending = "pending"  # "Ожидает отправки"
+    in_process = "in_process"  # "В процессе"
+    on_check = "on_check"  # "На проверке"
+    succeeded = "succeeded"  # "Отправлено"
+    failed = "failed"  # "Ошибка"
+
+
 class ActivityResponse(BaseModel):
     id: int = Field(description="id")
     key: str = Field(description="unique key")
@@ -31,7 +53,9 @@ class ActivityResponse(BaseModel):
 
 class SectionResponse(BaseModel):
     id: int = Field(description="id")
-    activity_key: Optional[str] = Field(description="Тип деятельности из activities.key")
+    activity_key: Optional[str] = Field(
+        description="Тип деятельности из activities.key"
+    )
     title: str = Field(description="Название")
     image_url: Optional[str] = Field(description="Картинка типа")
     recommended: bool = Field(description="Рекомендовано или нет")
@@ -60,7 +84,9 @@ class MedicalResearchResponse(BaseModel):
         "заявитель будет заблокирован"
     )
     type: MedResearchType = Field(description="Тип исследования")
-    block_type: MedResearchBlockType = Field(description="Тип группы исследований")
+    block_type: Optional[MedResearchBlockType] = Field(
+        description="Тип группы исследований"
+    )
     results: List[MedResearchResult] = Field(
         description="Справочник возможных" " результатов"
     )
@@ -105,6 +131,7 @@ class MedicalBookRequest(BaseModel):
     date: datetime_date = Field(description="Дата выдачи ЛМК")
     user_id: int = Field(description="Id гигтест клиента")
     activity_keys: list = Field(description="Ключи типов деятельности")
+    is_elmk: Optional[MedBookType] = Field(description="Тип мед. книжки")
 
 
 class UserExtended(ClientInfo):
@@ -137,6 +164,7 @@ class MedicalBookSearchResponse(BaseModel):
 
 class MedicalBookResponse(BaseModel):
     id: int = Field(description="Id ЛМК в гигтесте")
+    regnum: str = Field(description="Рег. номер ЛМК")
 
 
 # АТТЕСТАЦИЯ
@@ -150,6 +178,9 @@ class AttestationRequest(BaseModel):
     mb_date: datetime_date = Field(description="Дата выдачи ЛМК")
     attestation_number: int = Field(description="1 или 0 (первичная или периодическая)")
     user_id: int = Field(description="Id гигтест клиента")
+    is_elmk: MedBookType = Field(
+        default=MedBookType.paper, description="Тип мед. книжки"
+    )
 
 
 class AttestationResponse(BaseModel):
@@ -171,6 +202,9 @@ class MedicineCreateRequest(BaseModel):
     date_completion: Optional[datetime_date] = Field(
         description="Дата завершения медосмотра"
     )
+    order_point: List[str] = Field(
+        description="Пункт приказа 23./24./25./26. только 1 из этих"
+    )
 
 
 class MedicineUpdateRequest(BaseModel):
@@ -180,19 +214,21 @@ class MedicineUpdateRequest(BaseModel):
 
 
 class MedDirectionResultData(BaseModel):
-    result_id: str = Field(description="Id исследования в гигтест")
+    result_key: str = Field(description="Код исследования в гигтест")
     date_med_result: datetime_date = Field(description="Дата исследования")
     description: str = Field(description="Описание")
-    self_explored: int = Field(description="0/1 (проведено у нас или нет)")
+    self_explored: Optional[int] = Field(description="0 перенос, None проведено у нас)")
 
 
 class AdditionalMedDirectionResultRequest(MedDirectionResultData):
-    medical_research_id: int = Field(description="Id исследования в гигтест")
+    medical_research_key: str = Field(description="Код исследования в гигтест")
     medical_direction_id: int = Field(description="Id медобследования в гигтест")
 
 
 class MedicineCreateOrUpdateResponseResults(BaseModel):
-    medical_research_id: int = Field(description="Id исследования в гигтест")
+    medical_research_key: Optional[str] = Field(
+        description="Код исследования в гигтест"
+    )  # с 01.10.2025 не все исследования от гигтеста приходят с этим ключом
     id: int = Field(description="Id медобследования в гигтест")
 
 
@@ -231,11 +267,125 @@ class AdditionalMedDirectionResultResponse(BaseModel):
     id: int = Field(description="Id исследования в гигтест")
 
 
+# СЭМД - ПЕРЕДАЧА В РЭМД
+class SignCaseRequest(BaseModel):
+    sign: str = Field(description="Подпись в base64")
+
+
+class DocumentInfo(BaseModel):
+    local_num: str = Field(description="Уникальный номер документа")
+    gigtest_num: Optional[str] = Field(
+        description="Уникальный номер документа в Гигтест"
+    )
+    created_at: str = Field(description="Дата и время создания СЭМДа (Y-m-d H:M:S)")
+    kind: int = Field(description="Тип СЭМДа")
+    description: str = Field(description="Произвольное краткое описание")
+    content: str = Field(description="СЭМД в base64")
+
+
+class OrganizationInfo(BaseModel):
+    oid: str = Field(description="Согласно ФРМО")
+    name: str = Field(description="Согласно ФРМО")
+
+
+class PatientInfo(BaseModel):
+    firstname: str = Field(description="Имя пациента")
+    lastname: str = Field(description="Фамилия пациента")
+    patrname: str = Field(description="Отчество пациента")
+    birthdate: str = Field(description="Дата рождения (YYYY-MM-DD)")
+    gender: GenderType = Field(description="Пол")
+    snils: str = Field(description="СНИЛС")
+
+
+class DoctorInfo(BaseModel):
+    firstname: str = Field(description="Имя врача")
+    lastname: str = Field(description="Фамилия врача")
+    patrname: str = Field(description="Отчество врача")
+    snils: str = Field(description="СНИЛС врача")
+    position_id: int = Field(description="ID должности согласно справочнику")
+    speciality_id: int = Field(description="ID специальности согласно справочнику")
+
+
+class CreateCaseRequest(BaseModel):
+    document: DocumentInfo = Field(description="Информация о документе")
+    organization: OrganizationInfo = Field(description="Информация об организации")
+    patient: PatientInfo = Field(description="Информация о пациенте")
+    doctor: DoctorInfo = Field(description="Информация о враче")
+
+
+class CaseError(BaseModel):
+    message: str = Field(description="Сообщение об ошибке")
+    description: str = Field(description="Описание ошибки")
+
+
+class CaseResponse(BaseModel):
+    id: int = Field(description="ID случая")
+    status: SyncStatus = Field(description="Статус")
+    remd_regnumber: Optional[str] = Field(description="Номер ЭМД в РЭМД")
+    error: Optional[CaseError] = Field(description="Список ошибок при отправке СЭМДа")
+
+    # в доке и примере разный ключ, `status` вместо `sync_status`
+    @root_validator(pre=True)
+    def accept_status_or_sync_status(cls, values):
+        if "status" not in values and "sync_status" in values:
+            values["status"] = values["sync_status"]
+        return values
+
+
+# СОЗДАНИЕ ПАЦИЕНТА В ИЭМК
+class PatientIEMK(BaseModel):
+    id: int = Field(description="Внутренний идентификатор пациента")
+    first_name: str = Field(description="Имя пациента")
+    last_name: str = Field(description="Фамилия пациента")
+    patronymic: Optional[str] = Field(None, description="Отчество пациента")
+    birthday: str = Field(description="Дата рождения пациента (YYYY-MM-DD)")
+    gender: GenderType = Field(description="Пол пациента")
+    phone: Optional[str] = Field(None, description="Телефон пациента")
+    email: Optional[str] = Field(None, description="Email пациента")
+
+
+class AddressInfo(BaseModel):
+    street_address_line: str = Field(description="Улица, дом и квартира")
+
+
+class RegionInfo(BaseModel):
+    name: str = Field(description="Название региона")
+
+
+class CountryInfo(BaseModel):
+    name: str = Field(description="Название страны")
+
+
+class SnilsInfo(BaseModel):
+    number: str = Field(description="Номер СНИЛС")
+
+
+class PassportInfo(BaseModel):
+    series: Optional[str] = Field(None, description="Серия паспорта")
+    number: Optional[str] = Field(None, description="Номер паспорта")
+    issue_org_name: Optional[str] = Field(None, description="Орган, выдавший паспорт")
+
+
+class EditPatientRequest(BaseModel):
+    patient: PatientIEMK = Field(description="Информация о пациенте")
+    address: AddressInfo = Field(description="Адрес проживания пациента")
+    region: RegionInfo = Field(description="Регион проживания пациента")
+    country: CountryInfo = Field(description="Страна проживания пациента")
+    snils: SnilsInfo = Field(description="СНИЛС пациента")
+    passport: Optional[PassportInfo] = Field(description="Паспортные данные пациента")
+    organization: OrganizationInfo = Field(
+        description="Медицинская организация, направившая пациента"
+    )
+
+
+class EditPatientResponse(BaseModel):
+    message: str
+
+
 class GigtestService:
-    def __init__(self, client: SDKClient, url: HttpUrl, token: str):
+    def __init__(self, client: SDKClient, url: HttpUrl):
         self._client = client
         self._url = url
-        self._token = token
         # СПРАВОЧНИКИ
         self._get_activities_url = "/api/v2/activities"
         self._get_sections_url = "/api/v2/sections"
@@ -252,48 +402,58 @@ class GigtestService:
         # МЕДИЦИНА
         self._create_update_medicine_url = "/api/v2/medical-directions"
         self._additional_medicine_results_url = "/api/v2/medical-research-results"
+        # СЭМД
+        self._cases_url = "/api/v2/cases"
+        self._create_patient_url = "/api/v2/cases/create-patient"
+        self._update_patient_url = "/api/v2/cases/update-patient"
 
     # СПРАВОЧНИКИ
-    def activities(self) -> SDKResponse[List[ActivityResponse]]:
+    def activities(self, token: str) -> SDKResponse[List[ActivityResponse]]:
         return self._client.get(
-            self._full_url(self._get_activities_url),
+            self._full_url(self._get_activities_url, token),
             ActivityResponse,
             timeout=COMMON_GIGTEST_TIMEOUT,
         )
 
-    def sections(self) -> SDKResponse[List[SectionResponse]]:
+    def sections(self, token: str) -> SDKResponse[List[SectionResponse]]:
         return self._client.get(
-            self._full_url(self._get_sections_url),
+            self._full_url(self._get_sections_url, token),
             SectionResponse,
             timeout=COMMON_GIGTEST_TIMEOUT,
         )
 
-    def countries(self) -> SDKResponse[List[CountryResponse]]:
+    def countries(self, token: str) -> SDKResponse[List[CountryResponse]]:
         return self._client.get(
-            self._full_url(self._get_countries_url),
+            self._full_url(self._get_countries_url, token),
             CountryResponse,
             timeout=COMMON_GIGTEST_TIMEOUT,
         )
 
-    def medical_research(self) -> SDKResponse[List[MedicalResearchResponse]]:
+    def medical_research(
+        self, token: str
+    ) -> SDKResponse[List[MedicalResearchResponse]]:
         return self._client.get(
-            self._full_url(self._get_medical_researches_url),
+            self._full_url(self._get_medical_researches_url, token),
             MedicalResearchResponse,
             timeout=COMMON_GIGTEST_TIMEOUT,
         )
 
     # КЛИЕНТ
-    def search_client(self, query: SearchClientRequest) -> SDKResponse[ClientResponse]:
+    def search_client(
+        self, query: SearchClientRequest, token: str
+    ) -> SDKResponse[ClientResponse]:
         return self._client.get(
-            self._full_url(self._search_client_url),
+            self._full_url(self._search_client_url, token),
             ClientResponse,
             params=query.dict(),
             timeout=COMMON_GIGTEST_TIMEOUT,
         )
 
-    def create_client(self, query: CreateClientRequest) -> SDKResponse[ClientResponse]:
+    def create_client(
+        self, query: CreateClientRequest, token: str
+    ) -> SDKResponse[ClientResponse]:
         return self._client.post(
-            self._full_url(self._create_or_update_client_url),
+            self._full_url(self._create_or_update_client_url, token),
             ClientResponse,
             data=query.json(exclude_none=True),
             timeout=LONG_GIGTEST_TIMEOUT,
@@ -301,11 +461,11 @@ class GigtestService:
         )
 
     def update_client(
-        self, query: UpdateClientRequest, client_gig_test_id: int
+        self, query: UpdateClientRequest, client_gig_test_id: int, token: str
     ) -> SDKResponse[ClientResponse]:
         return self._client.put(
             self._full_url(
-                self._create_or_update_client_url + "/" + str(client_gig_test_id)
+                self._create_or_update_client_url + "/" + str(client_gig_test_id), token
             ),
             ClientResponse,
             data=query.json(exclude_none=True),
@@ -313,20 +473,22 @@ class GigtestService:
             headers={"Content-Type": "application/json"},
         )
 
-    def get_client(self, client_gig_test_id: int) -> SDKResponse[ClientResponse]:
+    def get_client(
+        self, client_gig_test_id: int, token: str
+    ) -> SDKResponse[ClientResponse]:
         return self._client.get(
             self._full_url(
-                self._create_or_update_client_url + "/" + str(client_gig_test_id)
+                self._create_or_update_client_url + "/" + str(client_gig_test_id), token
             ),
             ClientResponse,
             timeout=COMMON_GIGTEST_TIMEOUT,
         )
 
     # этот метод для ручного использования!
-    def delete_client(self, client_gig_test_id: int) -> SDKResponse[Empty]:
+    def delete_client(self, client_gig_test_id: int, token) -> SDKResponse[Empty]:
         return self._client.delete(
             self._full_url(
-                self._create_or_update_client_url + "/" + str(client_gig_test_id)
+                self._create_or_update_client_url + "/" + str(client_gig_test_id), token
             ),
             Empty,
             timeout=LONG_GIGTEST_TIMEOUT,
@@ -334,30 +496,30 @@ class GigtestService:
 
     # МЕДКНИЖКИ
     def search_medbook_when_other_department(
-        self, medbook_number: int
+        self, medbook_number: int, token
     ) -> SDKResponse[MedicalBookSearchResponseOther]:
         return self._client.get(
-            self._full_url(self._medbook_url_search),
+            self._full_url(self._medbook_url_search, token),
             MedicalBookSearchResponseOther,
             params={"number": medbook_number},
             timeout=COMMON_GIGTEST_TIMEOUT,
         )
 
     def search_medbook(
-        self, medbook_number: int
+        self, medbook_number: int, token: str
     ) -> SDKResponse[List[MedicalBookSearchResponse]]:
         return self._client.get(
-            self._full_url(self._medbook_url),
+            self._full_url(self._medbook_url, token),
             MedicalBookSearchResponse,
             params={"MedicalBooksSearch[number]": medbook_number},
             timeout=COMMON_GIGTEST_TIMEOUT,
         )
 
     def create_medbook(
-        self, query: MedicalBookRequest
+        self, query: MedicalBookRequest, token: str
     ) -> SDKResponse[MedicalBookResponse]:
         return self._client.post(
-            self._full_url(self._medbook_url),
+            self._full_url(self._medbook_url, token),
             MedicalBookResponse,
             data=query.json(exclude_none=True),
             timeout=LONG_GIGTEST_TIMEOUT,
@@ -365,10 +527,10 @@ class GigtestService:
         )
 
     def update_medbook(
-        self, query: MedicalBookRequest, medbook_gig_test_id: int
+        self, query: MedicalBookRequest, medbook_gig_test_id: int, token: str
     ) -> SDKResponse[MedicalBookResponse]:
         return self._client.put(
-            self._full_url(self._medbook_url + "/" + str(medbook_gig_test_id)),
+            self._full_url(self._medbook_url + "/" + str(medbook_gig_test_id), token),
             MedicalBookResponse,
             data=query.json(exclude_none=True),
             timeout=LONG_GIGTEST_TIMEOUT,
@@ -377,10 +539,10 @@ class GigtestService:
 
     # АТТЕСТАЦИЯ
     def create_attestation(
-        self, query: AttestationRequest
+        self, query: AttestationRequest, token: str
     ) -> SDKResponse[AttestationResponse]:
         return self._client.post(
-            self._full_url(self._attestation_url),
+            self._full_url(self._attestation_url, token),
             AttestationResponse,
             data=query.json(exclude_none=True),
             timeout=LONG_GIGTEST_TIMEOUT,
@@ -388,10 +550,10 @@ class GigtestService:
         )
 
     def update_attestation(
-        self, query: AttestationRequest, gigtest_att_id: int
+        self, query: AttestationRequest, gigtest_att_id: int, token: str
     ) -> SDKResponse[AttestationResponse]:
         return self._client.put(
-            self._full_url(self._attestation_url + "/" + str(gigtest_att_id)),
+            self._full_url(self._attestation_url + "/" + str(gigtest_att_id), token),
             AttestationResponse,
             data=query.json(exclude_none=True),
             timeout=LONG_GIGTEST_TIMEOUT,
@@ -399,30 +561,31 @@ class GigtestService:
         )
 
     # этот метод для ручного использования!
-    def delete_attestation(self, gigtest_att_id: int) -> SDKResponse[Empty]:
+    def delete_attestation(self, gigtest_att_id: int, token: str) -> SDKResponse[Empty]:
         return self._client.delete(
-            self._full_url(self._attestation_url + "/" + str(gigtest_att_id)),
+            self._full_url(self._attestation_url + "/" + str(gigtest_att_id), token),
             Empty,
             timeout=LONG_GIGTEST_TIMEOUT,
         )
 
     # МЕДИЦИНА
     def get_med_direction(
-        self, gigtest_med_direction_id: int
+        self, gigtest_med_direction_id: int, token: str
     ) -> SDKResponse[MedicineCreateResponseExtended]:
         return self._client.get(
             self._full_url(
-                self._create_update_medicine_url + "/" + str(gigtest_med_direction_id)
+                self._create_update_medicine_url + "/" + str(gigtest_med_direction_id),
+                token,
             ),
             MedicineCreateResponseExtended,
             timeout=COMMON_GIGTEST_TIMEOUT,
         )
 
     def create_med_direction(
-        self, query: MedicineCreateRequest
+        self, query: MedicineCreateRequest, token: str
     ) -> SDKResponse[MedicineCreateResponse]:
         return self._client.post(
-            self._full_url(self._create_update_medicine_url),
+            self._full_url(self._create_update_medicine_url, token),
             MedicineCreateResponse,
             data=query.json(exclude_none=True),
             timeout=LONG_GIGTEST_TIMEOUT,
@@ -430,11 +593,12 @@ class GigtestService:
         )
 
     def update_med_direction(
-        self, query: MedicineUpdateRequest, gigtest_med_direction_id: int
+        self, query: MedicineUpdateRequest, gigtest_med_direction_id: int, token: str
     ) -> SDKResponse[MedicineUpdateResponse]:
         return self._client.put(
             self._full_url(
-                self._create_update_medicine_url + "/" + str(gigtest_med_direction_id)
+                self._create_update_medicine_url + "/" + str(gigtest_med_direction_id),
+                token,
             ),
             MedicineUpdateResponse,
             data=query.json(exclude_none=True),
@@ -442,10 +606,13 @@ class GigtestService:
             headers={"Content-Type": "application/json"},
         )
 
-    def delete_med_direction(self, gigtest_med_direction_id: int) -> SDKResponse[Empty]:
+    def delete_med_direction(
+        self, gigtest_med_direction_id: int, token: str
+    ) -> SDKResponse[Empty]:
         return self._client.delete(
             self._full_url(
-                self._create_update_medicine_url + "/" + str(gigtest_med_direction_id)
+                self._create_update_medicine_url + "/" + str(gigtest_med_direction_id),
+                token,
             ),
             Empty,
             timeout=LONG_GIGTEST_TIMEOUT,
@@ -453,39 +620,93 @@ class GigtestService:
 
     # TODO выше метод get_med_direction объединить с этим, проверив апи
     def search_med_direction(
-        self, gigtest_med_direction_id: int
+        self, gigtest_med_direction_id: int, token: str
     ) -> SDKResponse[MedicineUpdateResponse]:
         return self._client.get(
             self._full_url(
-                self._create_update_medicine_url + "/" + str(gigtest_med_direction_id)
+                self._create_update_medicine_url + "/" + str(gigtest_med_direction_id),
+                token,
             ),
             MedicineUpdateResponse,
             timeout=COMMON_GIGTEST_TIMEOUT,
         )
 
     def delete_med_direction_result(
-        self, gigtest_med_direction_result_id: int
+        self, gigtest_med_direction_result_id: int, token: str
     ) -> SDKResponse[Empty]:
         return self._client.delete(
             self._full_url(
                 self._additional_medicine_results_url
                 + "/"
-                + str(gigtest_med_direction_result_id)
+                + str(gigtest_med_direction_result_id),
+                token,
             ),
             Empty,
             timeout=LONG_GIGTEST_TIMEOUT,
         )
 
     def add_additional_result_to_med_direction(
-        self, query: AdditionalMedDirectionResultRequest
+        self, query: AdditionalMedDirectionResultRequest, token: str
     ) -> SDKResponse[AdditionalMedDirectionResultResponse]:
         return self._client.post(
-            self._full_url(self._additional_medicine_results_url),
+            self._full_url(self._additional_medicine_results_url, token),
             AdditionalMedDirectionResultResponse,
             data=query.json(exclude_none=True),
             timeout=LONG_GIGTEST_TIMEOUT,
             headers={"Content-Type": "application/json"},
         )
 
-    def _full_url(self, url):
-        return f"{self._url}{url}?access-token={self._token}"
+    # СЭМД
+    def create_case(
+        self, query: CreateCaseRequest, token: str
+    ) -> SDKResponse[List[CaseResponse]]:
+        return self._client.post(
+            self._full_url(self._cases_url, token),
+            CaseResponse,
+            data=query.json(exclude_none=True),
+            timeout=LONG_GIGTEST_TIMEOUT,
+            headers={"Content-Type": "application/json"},
+        )
+
+    def sign_case(
+        self, case_id: int, query: SignCaseRequest, token: str
+    ) -> SDKResponse[List[CaseResponse]]:
+        return self._client.post(
+            self._full_url(f"{self._cases_url}/{case_id}/sign", token),
+            CaseResponse,
+            data=query.json(exclude_none=True),
+            timeout=LONG_GIGTEST_TIMEOUT,
+            headers={"Content-Type": "application/json"},
+        )
+
+    def get_case(self, case_id: int, token: str) -> SDKResponse[List[CaseResponse]]:
+        return self._client.get(
+            self._full_url(f"{self._cases_url}/{case_id}", token),
+            CaseResponse,
+            timeout=LONG_GIGTEST_TIMEOUT,
+        )
+
+    def create_patient(
+        self, query: EditPatientRequest, token: str
+    ) -> SDKResponse[EditPatientResponse]:
+        return self._client.post(
+            self._full_url(self._create_patient_url, token),
+            EditPatientResponse,
+            data=query.json(exclude_none=True),
+            timeout=LONG_GIGTEST_TIMEOUT,
+            headers={"Content-Type": "application/json"},
+        )
+
+    def update_patient(
+        self, query: EditPatientRequest, token: str
+    ) -> SDKResponse[EditPatientResponse]:
+        return self._client.post(
+            self._full_url(self._update_patient_url, token),
+            EditPatientResponse,
+            data=query.json(exclude_none=True),
+            timeout=LONG_GIGTEST_TIMEOUT,
+            headers={"Content-Type": "application/json"},
+        )
+
+    def _full_url(self, url, token):
+        return f"{self._url}{url}?access-token={token}"
