@@ -203,6 +203,7 @@ class PreRecordRequest(BaseModel):
     roistat_first: Optional[str]
     with_discount: bool = True
     gift_code: str = ""
+    external_id: Optional[str]
     initial_cost: Optional[int]
 
     class Config:
@@ -234,6 +235,7 @@ class PreRecordRequest(BaseModel):
             "pay_type": self.pay_type,
             "with_discount": self.with_discount,
             "gift_code": self.gift_code,
+            "external_id": self.external_id,
         }
         if self.initial_cost:
             data["initial_cost"] = self.initial_cost
@@ -351,6 +353,7 @@ class PreRecordResponse(BaseModel):
     client_uuid: Optional[str]
     email: Optional[str]
     gift_code: Optional[str]
+    external_id: Optional[str]
     place: int
     extra_fields: Optional[LabExtraFields]
     covid_fields: Optional[CovidFields]
